import logging

from hurry.filesize import size
from tabulate import tabulate
from termcolor import colored

from ioccheck.cli.formatters import Formatter
from ioccheck.services import MalwareBazaar

logger = logging.getLogger(__name__)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


class MalwareBazaarFormatter(Formatter):
    def __init__(self, service: MalwareBazaar):
        Formatter.__init__(self, service)

    @property
    def detections(self):

        detection_percent = self.service.detection_coverage * 100

        detection_count_string = f"{self.service.detection_count} engines ({detection_percent:.2g}%) detected this file.\n"

        if self.service.detection_count == 0:
            detection_count_string = colored(detection_count_string, "green")
        elif self.service.detection_count > 0:
            detection_count_string = colored(detection_count_string, "red")

        table = [["Antivirus", "Detected", "Result"]]

        for detection, result in self.service.detections.items():
            if result.get("category") == "malicious":
                malicious = colored("Yes", "red")
            else:
                malicious = colored("No", "green")

            table.append([detection, malicious, result.get("result")])

        return tabulate(table, headers="firstrow", tablefmt="fancy_grid")

    @property
    def tags(self):
        return ", ".join(self.servicetags) if self.service.tags else None

    @property
    def file_size(self):
        return size(self.service.file_size)

    @property
    def file_type(self):
        return f".{self.service.file_type}"

    @property
    def hashes(self):
        results = [[k, v] for k, v in self.service.hashes.items() if v]

        table = [["Type", "Hash"]]
        [table.append(result) for result in results]

        return tabulate(table, headers="firstrow", tablefmt="fancy_grid")



#!/usr/bin/env python
"""Module provides a base Service for implenting other services upon"""

from abc import ABC, abstractmethod
from typing import Optional


class Service(ABC):
    """Service base class for adding new threat intelligence services"""

    name: str

    def __new__(cls, *args):
        if not hasattr(cls, "name"):
            raise NotImplementedError(
                "'Service' subclasses should have a 'name' attribute"
            )
        if not hasattr(cls, "url"):
            raise NotImplementedError(
                "'Service' subclasses should have a 'url' attribute"
            )
        return object.__new__(cls)

    def __init__(self):
        pass

    @abstractmethod
    def _get_api_response(self, ioc, api_key: str) -> dict:
        pass

    @property
    def reputation(self) -> Optional[int]:
        pass

    def __str__(self):
        return self.name



import logging
from ipaddress import IPv4Address, IPv6Address
from typing import Union

import shodan
from backoff import expo, on_exception
from ratelimit import RateLimitException, limits

from ioccheck.services.service import Service

logger = logging.getLogger(__name__)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


class Shodan(Service):
    name = "shodan"
    url = "https://shodan.io/host/"

    def __init__(self, ip: Union[IPv4Address, IPv6Address], api_key: str):

        self.ip = ip
        self.response = self._get_api_response(ip, api_key)

        self.investigation_url = f"{self.url}/{ip}"

        self.location = self._get_location_data(self.response)
        self.hostnames = self._get_hostnames(self.response)
        self.tags = self._get_tags(self.response)
        self.vulns = self._get_vulns(self.response)

    @on_exception(expo, RateLimitException, max_tries=10)
    @limits(calls=15, period=60)
    def _get_api_response(
        self, ip: Union[IPv4Address, IPv6Address], api_key: str
    ) -> dict:
        client = shodan.Shodan(api_key)
        result = client.host(str(ip))
        return result

    def _get_tags(self, response) -> list:
        return response.get("data")[0].get("tags")

    def _get_location_data(self, response) -> dict:
        keys = [
            "region_code",
            "postal_code",
            "country_code",
            "city",
            "area_code",
            "country_name",
            "org",
            "isp",
            "asn",
        ]
        return {k: response.get(k) for k in keys}

    def _get_hostnames(self, response) -> list:
        return response.get("hostnames")

    def _get_vulns(self, response) -> list:
        return response.get("vulns")



#!/usr/bin/env python
from dataclasses import dataclass


class IOCType:
    pass


@dataclass
class HashType(IOCType):
    name: str
    regex: str

    def __str__(self) -> str:
        return self.name


@dataclass
class IPType(IOCType):
    name: str

    def __str__(self) -> str:
        return self.name


SHA256 = HashType(name="SHA256", regex=r"^[A-Fa-f0-9]{64}$")
SHA1 = HashType(name="SHA256", regex=r"^[A-Fa-f0-9]{40}$")
MD5 = HashType(name="MD5", regex=r"^[a-f0-9]{32}$")

IPv4 = IPType(name="IPv4")
IPv6 = IPType(name="IPv6")

hash_types = [SHA256, SHA1, MD5]
ip_types = [IPv4, IPv6]



#!/usr/bin/env/python
"""Module for the formatter base class"""

from ioccheck.services import Service


class Formatter:
    """Base class for creating human-friendly threat intelligence API output"""

    def __init__(self, service: Service):
        self.service = service



class InvalidHashException(Exception):
    pass


class InvalidIPException(Exception):
    pass


class NoConfiguredServicesException(Exception):
    pass


class InvalidCredentialsException(Exception):
    pass
